/*
 *
 * nucleo/image/processing/basic/Transform.cxx --
 *
 * Copyright (C) Nicolas Roussel
 *
 * See the file LICENSE for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <nucleo/config.H>

#include <nucleo/image/processing/basic/Transform.H>

#include <nucleo/image/encoding/Conversion.H>

namespace nucleo {

  // ------------------------------------------------------------------

  bool
  rotateImage(Image *img, bool clockwise) {
    unsigned int width=img->getWidth(), height=img->getHeight() ;
    if (!width || !height) return false ;

    if (!convertImage(img, Image::CONVENIENT)) return false ;

    unsigned int bytesPerPixel = img->getBytesPerPixel() ;
    int size = img->getSize() ;
    unsigned char *old_ptr = img->getData() ;
    int old_linewidth = width*bytesPerPixel ;
    unsigned char *new_ptr = Image::AllocMem(size) ;
    int new_linewidth = height*bytesPerPixel ;

    for (unsigned int y=0; y<height; ++y)
	 for (unsigned int x=0; x<width; ++x) {
	   int newx, newy ;
	   if (clockwise) {
		newx = height-1-y ;
		newy = x ;
	   } else {
		newx = y ;
		newy = width-1-x ;
	   }
	   unsigned char *src = old_ptr + y*old_linewidth + x*bytesPerPixel ;
	   unsigned char *dst = new_ptr + newy*new_linewidth + newx*bytesPerPixel ;
	   memcpy(dst, src, bytesPerPixel) ;
	 }

    img->setData(new_ptr, size, Image::FREEMEM) ;
    img->setDims(height, width) ;

    return true ;
  }

  // ------------------------------------------------------------------

  bool
  mirrorImage(Image *img, char hORv) {
    unsigned int width=img->getWidth(), height=img->getHeight() ;
    if (!width || !height) return false ;

    if (!convertImage(img, Image::CONVENIENT)) return false ;

    unsigned int bytesPerPixel = img->getBytesPerPixel() ;
    unsigned int size = img->getSize() ;
    unsigned char *mirror_ptr = Image::AllocMem(size) ;
    unsigned int linewidth = width*bytesPerPixel ;

    if (hORv=='h') {
	 unsigned char *src=img->getData(), *dst=mirror_ptr ;
	 for (unsigned int y=0; y<height; ++y) {
	   src += linewidth ;
	   for (unsigned int x=0; x<width; ++x) {
		src -= bytesPerPixel ;
		memcpy(dst, src, bytesPerPixel) ;
		dst += bytesPerPixel ;
	   }
	   src += linewidth ;
	 }
    } else if (hORv=='v') {
	 unsigned char *src=img->getData(), *dst=mirror_ptr+size-linewidth ;
	 for (unsigned int i=0; i<height; ++i) {
	   memcpy(dst, src, linewidth) ;
	   src += linewidth ;
	   dst -= linewidth ;
	 }
    }

    img->setData(mirror_ptr, size, Image::FREEMEM) ;
    return true ;
  }

  // ------------------------------------------------------------------

}
