/*
 *
 * tests/test-URI.cxx --
 *
 * Copyright (C) Nicolas Roussel
 *
 * See the file LICENSE for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <nucleo/core/URI.H>

#include <iostream>
#include <string>

using namespace nucleo ;

int
main(void) {
  const char *tests[] = {
	   "http://user:password@host:1212/path1/path2?query1=query2#frag",
	   "/first/second?user=nicolas+roussel&host=sgi3&time=now",
	   "//user@host/toto/tutu",
	   "/pub/python",
	   "index.html",
	   "/",
	   "?coucou=toto",
	   "http://:8001",
	   "mailto:user@host.domain",
	   "mcast://225.0.0.250:8123",
	   "mcast://<broadcast>:8080",
        "http://nansg1.in2p3.fr:3128/http://smaragd.can.nl:5555/video?zoom=4"
  } ;
  const int nbtests=12 ;

  URI *uri ;
  int i ;
  for (i=0; i<nbtests; i++) {
    uri = new URI(tests[i]) ;
    std::cout << std::endl << "-------------------------------------------------" << std::endl ;
    std::cout << tests[i] << std::endl << std::endl ;
    uri->debug(std::cout) ;
    delete uri ;
  }

  std::cout << std::endl << "-------------------------------------------------" ;
  std::cout << std::endl << "-------------------------------------------------" << std::endl ;

  const char *testsQuery[] = {
    "findme=30.0&zoom=2&length=50000",
    "zoom=2&findme=30&length=50000",
    "zoom=2&length=50000&findme=nicolas",
    "findme&zoom=2&length=50000",
    "zoom=2&notfindme&findme",
    "findmenot=2&length=50000&findme",
    "qslkdfjhsd",
    "findme&toto",
    "toto&findme"
  } ;
  const int nbtestsQuery=9 ;

  for (i=0; i<nbtestsQuery; i++) {
    std::cerr << "query: " << testsQuery[i] << " --> " ;
    std::string arg ;
    if (URI::getQueryArg(testsQuery[i], "findme", &arg))
	 std::cout << "findme='" << arg << "'" << std::endl ;
    else
	 std::cout << "findme not found" << std::endl ;
  }

}
